"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDiagnosticsForMissingNodes = exports.findParent = exports.isReference = exports.isDefinition = exports.range = exports.forEach = void 0;
const node_1 = require("vscode-languageserver/node");
/**
 * Recursively iterate over all nodes in a tree.
 *
 * @param node The node to start iterating from
 * @param callback The callback to call for each node. Return false to stop following children.
 */
function forEach(node, callback) {
    const followChildren = callback(node) !== false;
    if (followChildren && node.children.length) {
        node.children.forEach((n) => forEach(n, callback));
    }
}
exports.forEach = forEach;
function range(n) {
    return node_1.Range.create(n.startPosition.row, n.startPosition.column, n.endPosition.row, n.endPosition.column);
}
exports.range = range;
function isDefinition(n) {
    switch (n.type) {
        case 'variable_assignment':
        case 'function_definition':
            return true;
        default:
            return false;
    }
}
exports.isDefinition = isDefinition;
function isReference(n) {
    switch (n.type) {
        case 'variable_name':
        case 'command_name':
            return true;
        default:
            return false;
    }
}
exports.isReference = isReference;
function findParent(start, predicate) {
    let node = start.parent;
    while (node !== null) {
        if (predicate(node)) {
            return node;
        }
        node = node.parent;
    }
    return null;
}
exports.findParent = findParent;
function getDiagnosticsForMissingNodes(node) {
    const diagnostics = [];
    forEach(node, (node) => {
        if (node.isMissing()) {
            diagnostics.push(node_1.Diagnostic.create(range(node), `Syntax error: "${node.type}" missing`, node_1.DiagnosticSeverity.Warning, undefined, 'bash-language-server'));
        }
        return node.hasError();
    });
    return diagnostics;
}
exports.getDiagnosticsForMissingNodes = getDiagnosticsForMissingNodes;
//# sourceMappingURL=tree-sitter.js.map